#version 430 core
out vec4 FragColor;
  
in vec2 fTextCoords;

uniform sampler2D edgeTexture;
uniform int resolutionX;
uniform int resolutionY;


vec2 texSize = textureSize(edgeTexture,0);
float x = 2.0/texSize.x;
float y = 2.0/texSize.y;

void main()
{ 	
	vec2 samplePos[9] = vec2[](
		vec2(-x,	y), 
        vec2( 0.0f, y),
        vec2( x,	y),
        vec2(-x,	0.0f),
        vec2( 0.0f, 0.0f),
        vec2( x,	0.0f),
        vec2(-x,	-y),
        vec2( 0.0f, -y),
        vec2( x,	-y)
	);

	//sobel X kernel (horizontal edges)
	float sobelX[9] = float[](
		1, 2, 1,
		0, 0, 0,
       -1, -2,-1
    );

	//sobel Y kernel (vertical edges)
	float sobelY[9] = float[](
		1, 0, -1,
		2, 0, -2,
        1, 0, -1
    );
    

	//get part of texture that gets convolved with sobel kernels
	vec3 texturePart[9];
	for(int i = 0; i < 9; i++)
    {
       texturePart[i] = vec3(texture(edgeTexture, fTextCoords.xy + samplePos[i]));		
    }	
    
	//convolute kernel with image part
    vec3 resultX = (sobelX[0] * texturePart[0] + sobelX[1] * texturePart[1] + sobelX[2] * texturePart[2] + sobelX[6] * texturePart[6] + sobelX[7] * texturePart[7] + sobelX[8] * texturePart[8]);
	vec3 resultY = (sobelY[0] * texturePart[0] + sobelY[3] * texturePart[3] + sobelY[6] * texturePart[6] + sobelY[2] * texturePart[2] + sobelY[5] * texturePart[5] + sobelY[8] * texturePart[8]);

	vec3 resultSobel = sqrt((resultX * resultX) + (resultY * resultY));
	//vec3 resultSobel = resultX + resultY; //without square root
	
	FragColor = vec4(resultSobel, 1.0);
	
	//FragColor = texture(edgeTexture, fTextCoords); //debug without filter

}