#version 430 core
//shader for rendering scene with shadows
layout (location = 0) in vec3 positions;
layout (location = 1) in vec3 normals;
layout (location = 2) in vec2 vTextCoords;

out vec3 fPosition;
out vec3 fNormal;
out vec2 fTextCoords;
out vec4 fPositionLightSpace;

uniform mat4 model;
uniform mat4 modelInv; //inverse model matrix
uniform mat4 vp; //projection * view
uniform mat4 lightVpMatrix; //light's vp matrix, transform to light space (moon)


void main()
{
	fPosition = vec3(model * vec4(positions, 1.0)); //transform vertex position to world space, because lighting calculations in fragment shader in world space
    mat3 normalMatrix = mat3(transpose(modelInv));
	fNormal = normalMatrix * normals; //so normals stay perpendicular when scaled	
	fPositionLightSpace = lightVpMatrix * vec4(fPosition, 1.0); //position in light space (moon)
	fTextCoords = vTextCoords;

	gl_Position = vp * vec4(fPosition, 1.0);    
	
}